/*
   FNT - SQL-backend for PostgreSQL - function fetch_row

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <libpq-fe.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../include/global.h"

#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/sql.h"

#include "main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

#define SQL_LII_QUERY "SELECT currval('%s_id_seq')"

DLLEXPORT long long
last_inserted_id (fnt_sql_query_t * query, const char *table_name)
{
    fnt_sql_query_t *sql_query;
    fnt_sql_row_t *row;
    char *query_str;
    unsigned long long ret_value;

    char *escaped_table_name = fnt_sql_escape (query->connect, table_name);
    if (*escaped_table_name == '\0')
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Error escaping of table name"));
	g_free (escaped_table_name);
	return 0;
    }
    query_str = g_try_malloc (strlen (SQL_LII_QUERY) + strlen (escaped_table_name));
    if (query_str == NULL)
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Unable get last inserted ID from query"));
	g_free (escaped_table_name);
	return 0;
    }

    sprintf (query_str, SQL_LII_QUERY, escaped_table_name);
    g_free (escaped_table_name);

    sql_query = fnt_sql_query (query->connect, query_str);
    g_free (query_str);

    if (sql_query == NULL)
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Pgsql - fail to get last inserted ID"));
	return 0;
    }
    row = fnt_sql_fetchRow (sql_query);

    query_str = fnt_sql_getValueByFieldNumber (row, 0);
    if (query_str == NULL)
    {
	fnt_sql_rowClear (&row);
	fnt_sql_queryClear (&sql_query);
	return 0;
    }

    ret_value = atoll (query_str);

    fnt_sql_rowClear (&row);
    fnt_sql_queryClear (&sql_query);

    return ret_value;
}
